<?php

namespace Drupal\mailgroup_mailgun\Plugin\Mailgroup\Connection;

use Drupal\mailgroup\ConnectionBase;
use Mailgun\Mailgun as MailgunPHP;
use Mailgun\Message\MessageBuilder;
use Mailgun\Exception\HttpClientException;
use Mailgun\Message\Exceptions\TooManyRecipients;

/**
 * Basic server connection plugin.
 *
 * @Plugin(
 *   id = "mailgun",
 *   label = @Translation("Mailgun")
 * )
 */
class Mailgun extends ConnectionBase {

  /**
   * {@inheritdoc}
   */
  public function getFields() {
    $config = $this->configuration;

    $fields['domain'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Domain'),
      '#default_value' => isset($config['domain']) ? $config['domain'] : NULL,
    ];

    $fields['api_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('API key'),
      '#default_value' => isset($config['api_key']) ? $config['api_key'] : NULL,
    ];

    return $fields;
  }

  /**
   * {@inheritdoc}
   */
  public function testConnection() {
    $config = $this->configuration;
    $mailgun = MailgunPHP::create($config['api_key']);

    try {
      $mailgun->domains()->verify($config['domain']);
    }

    catch (HttpClientException $e) {
      return FALSE;
    }

    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function sendMessage(array $message) {
    $config = $this->configuration;

    $builder = new MessageBuilder();
    $builder->setFromAddress($message['sender']);
    $builder->setReplyToAddress($message['reply_to']);
    $builder->setSubject($message['subject']);
    $builder->setTextBody($message['body']);

    try {
      $builder->addToRecipient($message['to']);
    }

    catch (TooManyRecipients $e) {
      $this->logger->error($this->t('Unable to set to address. Too many recipients.'));
    }

    foreach ($message['emails'] as $email) {
      try {
        $builder->addBccRecipient($email);
      }

      catch (TooManyRecipients $e) {
        $this->logger->error($this->t('Unable to add BCC address. Too many recipients.'));
      }
    }

    try {
      $mailgun = MailgunPHP::create($config['api_key']);
      $mailgun->messages()->send($config['domain'], [
        'from' => $message['sender'],
        'to' => $message['to'],
        'bcc' => implode(', ', $message['emails']),
        'subject' => $message['subject'],
        'text' => $message['body'],
      ]);
    }

    catch (\Exception $e) {
      $this->logger->error($this->t('Unable to send message to @to', ['@to' => $message['to']]));
    }

  }

}
